// 2007/09/16 Hiroyuki Ogasawara
// vim:ts=4 sw=4:
/*!
	@file
	@date 2007/09/16
	@author	Hiroyuki Ogasawara
	@brief
		H쐬

	@sa http://dench.flatlib.jp/
	@sa http://wlog.flatlib.jp/
*/

//-----------------------------------------------------------------------------

#define		MAX_RANDOMMAP		256
#define		lengthStep			(1.0f/32.0f)

#define		__USE_LOOP__		1


cbuffer RandBuffer {
	float4	Random[ MAX_RANDOMMAP ];
};


//-----------------------------------------------------------------------------

Texture2D		InputTexture;
SamplerState	inputstate
{
	Filter= MIN_MAG_MIP_POINT;
	AddressU= Clamp;
	AddressV= Clamp;
};

//-----------------------------------------------------------------------------
static const float4	vertex[4]= {
	{	-1.0f,	 1.0f,	0.0f,	1.0f,	},
	{	 1.0f,	 1.0f,	0.0f,	1.0f,	},
	{	-1.0f,	-1.0f,	0.0f,	1.0f,	},
	{	 1.0f,	-1.0f,	0.0f,	1.0f,	},
};
static const float2	uv[4]= {
	{	0.0f,	0.0f,	},
	{	1.0f,	0.0f,	},
	{	0.0f,	1.0f,	},
	{	1.0f,	1.0f,	},
};

/*!
	generate vertex
*/
void VS_Main( uint vid : SV_VertexID,
			out float4 opos : SV_POSITION, out float2 ouv : TEXCOORD )
{
	opos= vertex[ vid ];
	ouv= uv[ vid ];
}


//-----------------------------------------------------------------------------

/*!
	copy only
*/
float4 PS_Copy( float4 Pos : SV_POSITION, float2 UV : TEXCOORD ) : SV_Target
{
	return	InputTexture.Sample( inputstate, UV );
}


//-----------------------------------------------------------------------------

float4 Rand( float4 pos )
{
	uint	index= (uint)( pos.z * 0.23f + pos.y * 0.31f + pos.x * 0.99f );
	return	Random[ index & (MAX_RANDOMMAP-1) ];
}

//-----------------------------------------------------------------------------

int isWall( float4 color )
{
	return	color.x >= 1.0f;
}

int isBlank( float4 color )
{
	return	color.x <= 0.001f;
}

int isReserve( float4 color )
{
	return	color.x <= 0.30f && color.x > 0.1f;
}

int isHead( float4 color )
{
	return	color.x <= 0.75f && color.x > 0.30f;
}

//-----------------------------------------------------------------------------

int isUp( float4 color )
{
	return	color.y <= 0.25f;
}

int isDown( float4 color )
{
	return	color.y <= 0.50f;
}

int isLeft( float4 color )
{
	return	color.y <= 0.75f;
}

//-----------------------------------------------------------------------------

int isUpS( float4 color )
{
	return	color.y <= 0.25f;
}

int isDownS( float4 color )
{
	return	color.y > 0.25f && color.y <= 0.50f;
}

int isLeftS( float4 color )
{
	return	color.y > 0.50f && color.y <= 0.75f;
}

int isRightS( float4 color )
{
	return	color.y > 0.75f;
}

//-----------------------------------------------------------------------------



float4 White( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	uint	x= (uint)pos.x;
	uint	y= (uint)pos.y;
	if( ((x|y) & 1) != 0 ){
		return	float4( 1, 1, 1, 1 );
	}
	if( Rand( pos ).x >= 0.997f ){
		return	float4( 0.5, Rand( pos ).y, Rand( pos ).w, 1 );
	}
	return	float4( 1, 1, 1, 1 );
}


const static float4	black_check_table[]= {
	{	 0.0f,	-1.0f,	-0.1f,	0.25f,	},	// Up
	{	 0.0f,	 1.0f,	0.25f,	0.50f,	},	// Down
	{	-1.0f,	 0.0f,	0.50f,	0.75f,	},	// Left
	{	 1.0f,	 0.0f,	0.75f,	1.00f,	},	// Right
};


float4 Black( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
#if __USE_LOOP__

	[unroll]
	for( int i= 0 ; i< 4 ; i++ ){
		float4	btable= black_check_table[i];
		float2	vec= pixsize.xy * btable.xy;
		float4	color_a= InputTexture.Sample( inputstate, uv - vec );
		if( isHead( color_a )
				&& color_a.y > btable.z && color_a.y <= btable.w ){	// isUpS ..
			float4	color_b= InputTexture.Sample( inputstate, uv + vec );
			if( isBlank( color_b ) ){
				return	float4( 0.25f, 0, 0, 0 );	// reserve
			}
		}
	}


#else

	float2	vec;
	float4	color_a;

	vec= float2( 0, -pixsize.y );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isDownS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		if( isBlank( color_b ) ){
			return	float4( 0.25f, 0, 0, 0 );	// reserve
		}
	}


	vec= float2( 0,  pixsize.y );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isUpS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		if( isBlank( color_b ) ){
			return	float4( 0.25f, 0, 0, 0 );	// reserve
		}
	}


	vec= float2( -pixsize.x, 0 );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isRightS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		if( isBlank( color_b ) ){
			return	float4( 0.25f, 0, 0, 0 );	// reserve
		}
	}

	vec= float2(  pixsize.x, 0 );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isLeftS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		if( isBlank( color_b ) ){
			return	float4( 0.25f, 0, 0, 0 );	// reserve
		}
	}
#endif

	return	float4( 0, 0, 0, 0 );
}



float4 Reserved( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
#if __USE_LOOP__
	[unroll]
	for( int i= 0 ; i< 4 ; i++ ){
		float4	btable= black_check_table[i];
		float2	vec= pixsize.xy * btable.xy;
		float4	color_a= InputTexture.Sample( inputstate, uv - vec );
		if( isHead( color_a )
				&& color_a.y > btable.z && color_a.y <= btable.w ){	// isUpS ..
			float2	vec2= pixsize.xy * btable.yx;
			float2	uv2= uv + vec;
			float4	color_b= InputTexture.Sample( inputstate, uv2 );
			float4	color_c= InputTexture.Sample( inputstate, uv2 + vec2 );
			float4	color_d= InputTexture.Sample( inputstate, uv2 - vec2 );
			if( isBlank( color_b )
							&& isBlank( color_c ) && isBlank( color_d ) ){
				color_a.z-= lengthStep;
				color_a.w= 1.0f;
				if( color_a.z <= 0.01f ){
					color_a.z= Rand( pos ).y;
					if( color_a.y > 0.50f ){
						color_a.y= Rand( pos ).z*0.500f;
					}else{
						color_a.y= Rand( pos ).z*0.5f + 0.5f;
					}
				}
				return	color_a;
			}
		}
	}
#else

	float2	vec;
	float2	vec2;
	float4	color_a;

	vec= float2( 0, -pixsize.y );
	vec2= float2( pixsize.x, 0 );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isDownS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		float4	color_c= InputTexture.Sample( inputstate, uv - vec + vec2 );
		float4	color_d= InputTexture.Sample( inputstate, uv - vec - vec2 );
		if( isBlank( color_b ) && isBlank( color_c ) && isBlank( color_d ) ){
			color_a.z-= lengthStep;
			color_a.w= 1.0f;
			if( color_a.z <= 0.01f ){
				color_a.z= Rand( pos ).y;
				if( color_a.y > 0.50f ){
					color_a.y= Rand( pos ).z*0.500f;
				}else{
					color_a.y= Rand( pos ).z*0.5f + 0.5f;
				}
			}
			return	color_a;
		}
	}


	vec= float2( 0,  pixsize.y );
	vec2= float2( pixsize.x, 0 );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isUpS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		float4	color_c= InputTexture.Sample( inputstate, uv - vec + vec2 );
		float4	color_d= InputTexture.Sample( inputstate, uv - vec - vec2 );
		if( isBlank( color_b ) && isBlank( color_c ) && isBlank( color_d ) ){
			color_a.z-= lengthStep;
			color_a.w= 1.0f;
			if( color_a.z <= 0.01f ){
				color_a.z= Rand( pos ).y;
				if( color_a.y > 0.50f ){
					color_a.y= Rand( pos ).z*0.500f;
				}else{
					color_a.y= Rand( pos ).z*0.5f + 0.5f;
				}
			}
			return	color_a;
		}
	}


	vec= float2( -pixsize.x, 0 );
	vec2= float2( 0, pixsize.y );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isRightS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		float4	color_c= InputTexture.Sample( inputstate, uv - vec + vec2 );
		float4	color_d= InputTexture.Sample( inputstate, uv - vec - vec2 );
		if( isBlank( color_b ) && isBlank( color_c ) && isBlank( color_d ) ){
			color_a.z-= lengthStep;
			color_a.w= 1.0f;
			if( color_a.z <= 0.01f ){
				color_a.z= Rand( pos ).y;
				if( color_a.y > 0.50f ){
					color_a.y= Rand( pos ).z*0.500f;
				}else{
					color_a.y= Rand( pos ).z*0.5f + 0.5f;
				}
			}
			return	color_a;
		}
	}

	vec= float2(  pixsize.x, 0 );
	vec2= float2( 0, pixsize.y );
	color_a= InputTexture.Sample( inputstate, uv + vec );
	if( isHead( color_a ) && isLeftS( color_a ) ){
		float4	color_b= InputTexture.Sample( inputstate, uv - vec );
		float4	color_c= InputTexture.Sample( inputstate, uv - vec + vec2 );
		float4	color_d= InputTexture.Sample( inputstate, uv - vec - vec2 );
		if( isBlank( color_b ) && isBlank( color_c ) && isBlank( color_d ) ){
			color_a.z-= lengthStep;
			color_a.w= 1.0f;
			if( color_a.z <= 0.01f ){
				color_a.z= Rand( pos ).y;
				if( color_a.y > 0.50f ){
					color_a.y= Rand( pos ).z*0.500f;
				}else{
					color_a.y= Rand( pos ).z*0.5f + 0.5f;
				}
			}
			return	color_a;
		}
	}
#endif

	return	float4( 0, 0, 0, 0 );
}




float4 MoveAction( float2 vec,
			float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	float4	color_a= InputTexture.Sample( inputstate, uv + vec );
	float4	color_b= InputTexture.Sample( inputstate, uv + vec*2 );

//	if( !isBlank( color_a ) )
	if( isWall( color_a ) || isHead( color_a ) )
	{
		color_00= float4( 1, 1, 1, 1 );	//  wall
	}else if( color_00.w <= 0.01f ){
		color_00= float4( 1, 1, 1, 1 );	//  wall
	}else if( !isBlank( color_b ) ){
		if( color_00.y > 0.50f ){
			color_00.y= Rand( pos ).z*0.499f;
			color_00.w-= 0.05f;
		}else{
			color_00.y= Rand( pos ).z*0.5f + 0.5f;
			color_00.w-= 0.05f;
		}
	}else{
		color_00.w-= 0.01f;
	}

	return	color_00;
}

float4 Up( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	return	MoveAction( float2( 0, -pixsize.y ), color_00, pos, uv, pixsize );
}


float4 Down( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	return	MoveAction( float2( 0,  pixsize.y ), color_00, pos, uv, pixsize );
}


float4 Left( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	return	MoveAction( float2( -pixsize.x, 0 ), color_00, pos, uv, pixsize );
}


float4 Right( float4 color_00, float4 pos, float2 uv, float2 pixsize )
{
	return	MoveAction( float2(  pixsize.x, 0 ), color_00, pos, uv, pixsize );
}



/*!
	@brief
		maze update
*/
float4 PS_Maze(
			noperspective float4 Pos : SV_POSITION,
			noperspective float2 UV : TEXCOORD ) : SV_Target
{
	float2	pixsize;
	InputTexture.GetDimensions( pixsize.x, pixsize.y );
	pixsize= 1/pixsize;
	Pos.z= Pos.y * pixsize.x + Pos.x;

	float4	color_00= InputTexture.Sample( inputstate, UV );
	float4	ocolor;
	if( isWall( color_00 ) ){
		ocolor= White( color_00, Pos, UV, pixsize );
	}else if( isBlank( color_00 ) ){
		ocolor= Black( color_00, Pos, UV, pixsize );
	}else if( isReserve( color_00 ) ){
		ocolor= Reserved( color_00, Pos, UV, pixsize );
	}else{
		if( isUp( color_00 ) ){
			ocolor= Up( color_00, Pos, UV, pixsize );
		}else if( isDown( color_00 ) ){
			ocolor= Down( color_00, Pos, UV, pixsize );
		}else if( isLeft( color_00 ) ){
			ocolor= Left( color_00, Pos, UV, pixsize );
		}else{
			ocolor= Right( color_00, Pos, UV, pixsize );
		}
	}

	return	ocolor;
}

//-----------------------------------------------------------------------------

VertexShader	vsmain= CompileShader( vs_4_0, VS_Main() );

technique10 Maze
{
	pass P0
	{
		SetVertexShader( vsmain );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PS_Maze() ) );
	}
}

technique10 Copy
{
	pass P0
	{
		SetVertexShader( vsmain );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PS_Copy() ) );
	}
}


//-----------------------------------------------------------------------------

