// 2007/09/16 Hiroyuki Ogasawara
// vim:ts=4 sw=4:
/*!
	@file
	@date	2007/09/15
	@author	Hiroyuki Ogasawara
	@brief

	H

	@sa http://dench.flatlib.jp/
	@sa http://wlog.flatlib.jp/
*/


#include	"whsample.h"


#define		_APP_TITLE_		TEXT("Wheel Handle ss03")


class Sample : public whSample {
public:
	enum {
		WINDOW_SIZE_WIDTH	= 512,
		WINDOW_SIZE_HEIGHT	= 512,
		MAX_RANDOM			= 256,
		MAX_RANDOM_FLOAT	= MAX_RANDOM *4,
	};
	enum {
		RENDERBUFFER_COUNT	= 2,
	};
private:
	ID3D10Effect*				iEffect;
	ID3D10InputLayout*			iInputLayout;
	ID3D10ShaderResourceView*	iTextureBuffer[RENDERBUFFER_COUNT];
	ID3D10RenderTargetView*		iRenderBuffer[RENDERBUFFER_COUNT];
	ID3D10DepthStencilState*	iDepthDisable;
	int		Src;
	int		Dest;
	UINT	RTWidth;
	UINT	RTHeight;
public:
	void	InitShader();
	void	ReleaseShader();
	void	Render();
	Sample() :
		iEffect( NULL ),
		iInputLayout( NULL ),
		iDepthDisable( NULL ),
		Src( 0 ),
		Dest( 1 ),
		RTWidth( 0 ),
		RTHeight( 0 )
	{
		for( int i= 0 ; i< RENDERBUFFER_COUNT ; i++ ){
			iTextureBuffer[i]= NULL;
			iRenderBuffer[i]= NULL;
		}
		InitDevice( TRUE, WINDOW_SIZE_WIDTH, WINDOW_SIZE_HEIGHT, _APP_TITLE_ );
		InitShader();
	}
	~Sample()
	{
		ReleaseShader();
		ReleaseDevice();
	}
};



/*!
	@brief
*/
void
Sample::InitShader()
{
	// Shader ̓ǂݍ
	ID3D10Blob*	iblob= NULL;
	if( FAILED( D3DX10CreateEffectFromFile(
				TEXT("maze.fx"),
				NULL, NULL, "fx_4_0", 0, 0, iDevice,
				NULL, NULL, &iEffect, &iblob, NULL ) ) ){
		OutputDebugStringA( reinterpret_cast<char*>(
									iblob->GetBufferPointer() ) );
		ZRELEASE( iblob );
		return;
	}


	// f[^̓ǂݍ
	D3DX10_IMAGE_LOAD_INFO	info;
	memset( &info, 0, sizeof(D3DX10_IMAGE_LOAD_INFO) );
	info.MipLevels= 1;
	info.Usage= D3D10_USAGE_DEFAULT;
	info.BindFlags= D3D10_BIND_SHADER_RESOURCE|D3D10_BIND_RENDER_TARGET;
	info.Format= DXGI_FORMAT_R8G8B8A8_UNORM;
	info.Filter= D3DX10_FILTER_NONE;
	info.MipFilter= D3DX10_FILTER_NONE;

	for( int i= 0 ; i< RENDERBUFFER_COUNT ; i++ ){
		// Load
		D3DX10CreateShaderResourceViewFromFile( iDevice,
				TEXT("initdata.png"),
				&info, NULL, &iTextureBuffer[i], NULL );

		// RenderTargetView ɕϊ
		ID3D10Resource*	riResource;
		iTextureBuffer[i]->GetResource( &riResource );
		D3D10_RENDER_TARGET_VIEW_DESC	rtvdesc;
		memset( &rtvdesc, 0, sizeof(D3D10_RENDER_TARGET_VIEW_DESC) );
		rtvdesc.Format= DXGI_FORMAT_R8G8B8A8_UNORM;
		rtvdesc.ViewDimension= D3D10_RTV_DIMENSION_TEXTURE2D;
		iDevice->CreateRenderTargetView( riResource,
										&rtvdesc, &iRenderBuffer[i] );
		// TCYQ
		D3D10_TEXTURE2D_DESC	t2ddesc;
		reinterpret_cast<ID3D10Texture2D*>( riResource )->GetDesc( &t2ddesc );
		RTWidth= t2ddesc.Width;
		RTHeight= t2ddesc.Height;
		ZRELEASE( riResource );
	}



	// Xe[g̍쐬
	D3D10_DEPTH_STENCIL_DESC	dssdesc;
	memset( &dssdesc, 0, sizeof(D3D10_DEPTH_STENCIL_DESC) );
	iDevice->CreateDepthStencilState( &dssdesc, &iDepthDisable );

	// 
	srand( GetTickCount() );

}


/*!
	@brief
		I
*/
void
Sample::ReleaseShader()
{
	if( iDevice ){
		iDevice->ClearState();
	}

	ZRELEASE( iDepthDisable );
	ZRELEASE( iEffect );
	for( int i= 0 ; i< RENDERBUFFER_COUNT ; i++ ){
		ZRELEASE( iTextureBuffer[i] );
		ZRELEASE( iRenderBuffer[i] );
	}
}


/*!
	@brief
		`惋[v
*/
void
Sample::Render()
{
	D3D10_VIEWPORT	viewport= { 0, 0, 0, 0, 0.0f, 1.0f };
	iDevice->OMSetDepthStencilState( iDepthDisable, 0 );
	iDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP );



	// e[u
	const float	randscale= 1.0f/32767.0f;
	float	RandomMap[ MAX_RANDOM_FLOAT ];
	for( int i= 0 ; i< MAX_RANDOM_FLOAT ; i++ ){
		RandomMap[i]= (rand() & 0x7fff) * randscale;
	}
	iEffect->GetVariableByName( "Random" )->AsVector()->
						SetFloatVectorArray( RandomMap, 0, MAX_RANDOM );


	// H̍XV
	viewport.Width= RTWidth;
	viewport.Height= RTHeight;
	iDevice->RSSetViewports( 1, &viewport );
	iEffect->GetVariableByName( "InputTexture" )->AsShaderResource()->
								SetResource( iTextureBuffer[Src] );
	iEffect->GetTechniqueByName( "Maze" )->GetPassByIndex( 0 )->Apply( 0 );
	iDevice->OMSetRenderTargets( 1, &iRenderBuffer[Dest], NULL );
	iDevice->Draw( 4, 0 );


	// t[obt@ɓ]
	viewport.Width= WINDOW_SIZE_WIDTH;
	viewport.Height= WINDOW_SIZE_HEIGHT;
	iDevice->RSSetViewports( 1, &viewport );
	iDevice->OMSetRenderTargets( 1, &iRenderTargetView, NULL );
	iEffect->GetVariableByName( "InputTexture" )->AsShaderResource()->
								SetResource( iTextureBuffer[Dest] );
	iEffect->GetTechniqueByName( "Copy" )->GetPassByIndex( 0 )->Apply( 0 );
	iDevice->Draw( 4, 0 );


	// Flip
	iSwapChain->Present( 0, 0 );
	Dest= Src;
	Src^= 1;
}




/*!
	@brief
		蔲main
*/
int APIENTRY
WinMain( HINSTANCE hinstance, HINSTANCE hpinstance, LPSTR cmdline, int show )
{
	Sample*	riSample= new Sample();


	for(; riSample->WaitMessage() ;){
		riSample->Render();
		Sleep( 1 );
	}


	delete riSample;
	return	0;
}

