// 2007/09/16 Hiroyuki Ogasawara
// vim:ts=4 sw=4:
/*!
	@file
	@date	2007/09/16
	@author	Hiroyuki Ogasawara
	@brief

	@sa http://dench.flatlib.jp/
	@sa http://wlog.flatlib.jp/
*/


#include	<windows.h>
#include	<d3d10.h>
#include	<d3dx10.h>

#include	"whsample.h"



whSample::whSample() :
		iSwapChain( NULL ),
		iDevice( NULL ),
		iRenderTargetView( NULL ),
		iDepthStencilView( NULL ),
		hWnd( NULL )
{
}


whSample::~whSample()
{
	ReleaseDevice();
}


/*!
	@param[in]	winmode
	@param[in]	width
	@param[in]	height
	@brief
		蔲

	R[hZ邱ƍŗDŃG[`FbN܂B
*/
void
whSample::InitDevice( int winmode, DWORD width, DWORD height, LPCTSTR title )
{
	// EBhE쐬
	HINSTANCE	hinstance= (HINSTANCE)GetModuleHandle( NULL );
	WNDCLASS	wc;
	wc.style= CS_HREDRAW|CS_VREDRAW;
	wc.lpfnWndProc= WindowExec;
	wc.cbClsExtra= 0;
	wc.cbWndExtra= 0;
	wc.hInstance= hinstance;
	wc.hIcon= NULL;
	wc.hCursor= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground= (HBRUSH)GetStockObject( BLACK_BRUSH );
	wc.lpszMenuName= NULL;
	wc.lpszClassName= title;
	RegisterClass( &wc );

	RECT	rect;
	SetRect( &rect, 0, 0, width, height );
	AdjustWindowRectEx( &rect, WS_OVERLAPPEDWINDOW, FALSE, WS_EX_APPWINDOW );

	hWnd= CreateWindowEx( WS_EX_APPWINDOW,
					title, title,
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, CW_USEDEFAULT,
					rect.right - rect.left,
					rect.bottom - rect.top,
					NULL, NULL, hinstance, NULL );
	ShowWindow( hWnd, SW_SHOWNORMAL );
	UpdateWindow( hWnd );





	// Device ̍쐬
	DXGI_SWAP_CHAIN_DESC	swapdesc= {
		{
			width, height,
			{	60,	1,	},
			DXGI_FORMAT_R8G8B8A8_UNORM,
			DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED,
			DXGI_MODE_SCALING_UNSPECIFIED,
		},
		{	1,	0,	},	// sample
		DXGI_USAGE_RENDER_TARGET_OUTPUT,
		1,
		hWnd,
		winmode,
		DXGI_SWAP_EFFECT_DISCARD,
		0
	};
	D3D10CreateDeviceAndSwapChain(
			NULL,
			D3D10_DRIVER_TYPE_HARDWARE,
			NULL,
			D3D10_CREATE_DEVICE_DEBUG,
			D3D10_SDK_VERSION,
			&swapdesc,
			&iSwapChain,
			&iDevice
		);


	// FrameBuffer ̍쐬
	ID3D10Texture2D*	riBackBuffer= NULL;
	iSwapChain->GetBuffer( 0, __uuidof( ID3D10Texture2D ),
							reinterpret_cast<void**>( &riBackBuffer ) );
	iDevice->CreateRenderTargetView(
							riBackBuffer, NULL, &iRenderTargetView );
	ZRELEASE( riBackBuffer );

	ID3D10Texture2D*	riTexture= NULL;
	D3D10_TEXTURE2D_DESC	tex2ddesc= {
		width, height, 1, 1,	// mip, array
		DXGI_FORMAT_D24_UNORM_S8_UINT,
		{	1,	0,	},	// sample
		D3D10_USAGE_DEFAULT,
		D3D10_BIND_DEPTH_STENCIL,
		0,	// cpu
		0	// misc
	};
	iDevice->CreateTexture2D( &tex2ddesc, NULL, &riTexture );

	D3D10_DEPTH_STENCIL_VIEW_DESC	vdesc= {
		DXGI_FORMAT_D24_UNORM_S8_UINT,
		D3D10_DSV_DIMENSION_TEXTURE2D
	};
	iDevice->CreateDepthStencilView( riTexture, &vdesc, &iDepthStencilView );
	ZRELEASE( riTexture );
	iDevice->OMSetRenderTargets( 1,
						&iRenderTargetView, iDepthStencilView );


	// Viewport ̐ݒ
	D3D10_VIEWPORT	viewport= {
		0, 0, width, height, 0.0f, 1.0f
	};
	iDevice->RSSetViewports( 1, &viewport );
}



/*!
	@brief
		\[XƃC^[tF[X̊J
*/
void
whSample::ReleaseDevice()
{
	ZRELEASE( iDepthStencilView );
	ZRELEASE( iDevice );
	ZRELEASE( iSwapChain );
}


/*!
	@brief
		EBhE Callback
*/
LRESULT CALLBACK
whSample::WindowExec( HWND hwnd, UINT mes, WPARAM wparam, LPARAM lparam )
{
	switch( mes ){
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return	FALSE;
	}
	return	DefWindowProc( hwnd, mes, wparam, lparam );
}



/*!
	@brief
		Cxg
*/
int
whSample::WaitMessage()
{
	for( MSG msg ; PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ;){
		if( msg.message == WM_QUIT ){
			return	FALSE;
		}
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return	TRUE;
}


